// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "display/TextureRenderTarget.hpp"
#include "scene/components/display/Camera.hpp"

namespace Dawn {
  class SubSceneCameraAlign : public SceneItemComponent {
    protected:
      TextureRenderTarget *renderTarget = nullptr;
      Camera *camera = nullptr;

      void onRenderTargetResize(RenderTarget *target, float_t w, float_t h);
      void onCameraResize(float_t w, float_t h);

      /**
       * Realigns the camera to match the render target quad.
       */
      void realign();

    public:
      /**
       * Create the sub scene camera align component. This will align a camera
       * to match a render target quad (Refer SimpleRenderTargetQuad)
       * 
       * @param item Scene Item that this component belongs to.
       */
      SubSceneCameraAlign(SceneItem *item);

      /**
       * Set the render target for this alignment to use.
       * 
       * @param renderTarget Render target to align to.
       */
      void setRenderTarget(TextureRenderTarget *renderTarget);

      /**
       * Sets the camera that will be aligned.
       * 
       * @param camera Camera to align.
       */
      void setCamera(Camera *camera);

      void onStart() override;
      void onDispose() override;
  };
}